import parseurl from 'parseurl'
import { createPath } from '../http.js'
import logger from '../logger.js'
import * as proxy from '../proxy.js'

export default function ({ urls }) {
  return async function (req, socket, head) {
    socket.on('error', (err) => logger.error(`Socket error on upgrade: ${err.stack}`))
    const { path } = parseurl(req)
    for (const [condition, config] of urls) {
      const { hostname, protocol, port, changeOrigin = false, secure } = config
      if (path.startsWith(condition)) {
        logger.info(`Websocket at "${path}" is handled by vite-plugin-proxy`)
        const headers = {}
        // http2-proxy does a changeOrigin as a default
        if (changeOrigin === true) {
          headers.host = port ? `${hostname}:${port}` : hostname
        } else {
          headers.host = req.headers[':authority'] || req.headers.host
        }
        try {
          return await proxy.ws(req, socket, head, { hostname, protocol, port, headers, path: createPath(path, config), secure })
        } catch (err) {
          logger.error(`Upgrade of websocket at "${path}" failed: ${err.stack}`)
          return
        }
      }
    }
  }
}
